/* -*- Mode: C++ -*-
 * $Id: FCFooDecision.cc,v 1.21 2001/10/14 21:21:43 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooDecision.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.21 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/10/14 21:21:43 $
 *
 *
 * HISTORY
 *
 * $Log: FCFooDecision.cc,v $
 * Revision 1.21  2001/10/14 21:21:43  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.20  2001/10/14 14:41:42  frehe
 * Added a sample communication between the agent and itself.
 *
 * Revision 1.19  2001/09/26 13:05:37  frehe
 * Changed the order of include files to make circular updates less likely.
 *
 * Revision 1.18  2001/09/01 23:55:08  frehe
 * Made minor changes.
 *
 * Revision 1.17  2001/08/30 22:08:26  frehe
 * Wrote two simple decision makers that goes to the ball and scores.
 *
 * Revision 1.16  2001/08/29 16:31:19  frehe
 * Added some exception handling.
 *
 * Revision 1.15  2001/08/29 14:16:27  frehe
 * Made a simpler decision maker to avoid skills (until I get them to work).
 *
 * Revision 1.14  2001/08/29 08:38:34  petan117
 * Unreachable code found, marked with a FIXME
 *
 * Revision 1.13  2001/08/17 07:15:16  frehe
 * Made changes to compile RoboSoc with RCCParser-0.10.
 *
 * Revision 1.12  2001/01/08 13:41:40  frehe
 * Fixed a minor bug in FCFooDecision.
 *
 * Revision 1.11  2001/01/08 13:27:45  frehe
 * Fixed a minor bug in FCFooDecision.
 *
 * Revision 1.10  2000/11/08 09:41:10  frehe
 * Updated the FCFooDecision to use the Score skill.
 *
 * Revision 1.9  2000/11/02 16:33:45  frehe
 * Minor updates.
 *
 * Revision 1.8  2000/10/24 15:15:46  frehe
 * Made minor updates and tests.
 *
 * Revision 1.7  2000/10/19 00:05:35  frehe
 * Made minor changes.
 *
 * Revision 1.6  2000/10/18 23:07:11  frehe
 * Added a say message and a display of them.
 *
 * Revision 1.5  2000/10/11 18:40:29  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.4  2000/10/10 11:55:37  frehe
 * Fixed FCFooDecision to work with the new skills.
 *
 * Revision 1.3  2000/10/08 17:34:11  frehe
 * Fixed FCFoo.
 *
 * Revision 1.2  2000/09/05 21:12:55  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 * Revision 1.1.1.1  2000/08/31 13:26:05  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FCFooDecision.cc
 * Implementation of the class FCFooDecision.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.21 $
 * $Author: frehe $
 * $Date: 2001/10/14 21:21:43 $
 */


#include "FCFooDecision.h"

#include <unistd.h>
#include <errno.h>

#include "ViewManager.h"
#include "ParameterView.h"
#include "GameView.h"
#include "CommandView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "PlayerView.h"
#include "WorldStateView.h"
#include "view_constants.h"
#include "ActuatorInterface.h"
#include "InitCommand.h"
#include "MoveCommand.h"
#include "DashCommand.h"
#include "KickCommand.h"
#include "TurnCommand.h"
#include "TurnNeckCommand.h"
#include "SayCommand.h"
#include "FindBall.h"
#include "GotoBallAvoidingObstacles.h"
#include "MoveToPositionWithBall.h"
#include "Score.h"
#include "rs_assert.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE


FCFooDecision::FCFooDecision(ActuatorInterface* const actuator_interface)
  : Decision(actuator_interface),
    viewManager(ViewManager::instance()),
    parameterView(NULL),
    gameView(NULL),
    commandView(NULL),
    markerView(NULL),
    ballView(NULL),
    agentView(NULL),
    playerView(NULL),
    worldStateView(NULL),
    serverAlive(true),
    aural_data(0)
#ifdef RS_USE_DISPLAY
  , theDisplay()
#endif
#ifdef RS_USE_COACH_DEBUGGING
  , coachView(NULL)
#endif
#ifdef RS_USE_OWN_LOGFILE
  , logFile("FCFoo.log")
#endif
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(commandView, COMMAND_VIEW_ID);
  getView(markerView, MARKER_VIEW_ID, parameterView);
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(playerView, PLAYER_VIEW_ID);
  getView(worldStateView, WORLD_STATE_VIEW_ID);
  
#ifdef RS_USE_COACH_DEBUGGING
  getView(coachView, COACH_VIEW_ID);
#endif
}


FCFooDecision::~FCFooDecision()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(COMMAND_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
#ifdef RS_USE_COACH_DEBUGGING
  viewManager->releaseView(COACH_VIEW_ID);
#endif
}


ServerCommand* FCFooDecision::initAgent()
{
  RS_REQUIRE( parameterView != NULL );
  InitCommand* init_cmd = new InitCommand(parameterView->CP_team_name,
					  parameterView->SP_version,
					  parameterView->CP_goalie);
  return init_cmd;
}


bool FCFooDecision::start()
{
  RS_REQUIRE( initialized == true );
  try {
    while ( serverAlive )
      try {
	if ( pause() == -1 && errno != EINTR )
	  RS_ERROR("Something went wrong in pause!");
      } catch (RSException &excpt) {
	std::cerr << "Caught a RoboSoc exception: " << excpt.What() << std::endl;
	return 1;
      } catch (std::exception &excpt) {
	std::cerr << "Caught a standard exception: " << excpt.what() << std::endl;
	return 1;
      } catch (...) {
	std::cerr << "Caught an unknown exception: " << std::endl;
	return 1;
      }
    return true;
  } catch (RSException &excpt) {
    std::cerr << "Caught a RoboSoc exception: " << excpt.What() << std::endl;
    return 1;
  } catch (std::exception &excpt) {
    std::cerr << "Caught a standard exception: " << excpt.what() << std::endl;
    return 1;
  } catch (...) {
    std::cerr << "Caught an unknown exception: " << std::endl;
    return 1;
  }
}


bool FCFooDecision::decide()
{
  static int say_counter = 10;
  
  // A primitive version of a decision maker that goes to the ball and scores.
  ServerCommand* scmd = 0;
  AngleDegUnknown ball_dir = ballView->getRelativeBallDirection(NOW);
  if ( ball_dir.isUnknown() ) {
    scmd = new TurnCommand(30);
  } else if ( abs(ball_dir) > 10 ) {
    scmd = new TurnCommand(ball_dir.getAngle());
  } else if ( gameView->getPlayMode() != PM_BEFORE_KICK_OFF ) {
    if ( ballView->getBallDistance(NOW) > 1 ) {
      scmd = new DashCommand(50);
    } else {
      PointUnknown ball_pos = ballView->getBallPosition(NOW);
      AngleDegUnknown goal_dir = markerView->getTheirGoal(NOW).getAbsoluteDirectionFrom(ball_pos);
      AngleDegUnknown body_dir = agentView->getBodyDirection(NOW);
      if ( goal_dir.isKnown() ) {
	if ( body_dir.isKnown() ) {
	  scmd = new KickCommand(100, goal_dir.getAngle() - body_dir.getAngle());
	} else {
	  scmd = new KickCommand(100, goal_dir.getAngle());
	}
      } else {
	scmd = new KickCommand(100, 0);
      }
    }
  } else {
    scmd = new TurnCommand(ball_dir.getAngle());
  }
  
  if ( scmd != 0 ) {
    actuatorInterface->addToPlan(scmd);
    delete scmd;
  }

  if ( say_counter == 0 ) {
    actuatorInterface->addToPlan(&SayCommand("ThickAIB 1 1 0 61 -15 37 -69 175 89 PP 0 8 90 -72 60 0 0 0 10 89 -4 -7 0 0 0 11 89 -6 5 0 0 1 4 87 160 1 0 0 1 5 85 222 200 0 0 1 6 88 86 -54 0 0 1 7 88 93 -1 0 0 1 8 88 86 49 0 0 1 10 79 282 -2 0 0 1 9 82 231 -198 0 0"));
    say_counter = 9;
  } else {
    --say_counter;
  }
  
  /*
  static FindBall findBall;
  static GotoBallAvoidingObstacles gotoBallAvoidingObstacles;
  static MoveToPositionWithBall moveToPositionWithBall(Point(55,0));
  static Score score;

  Plan plan;

  if ( score.applicable() ) {
    plan = score.generatePlan();
  }
  else if ( moveToPositionWithBall.applicable() ) {
    plan = moveToPositionWithBall.generatePlan();
  }
  else if ( gotoBallAvoidingObstacles.applicable() ) {
    plan = gotoBallAvoidingObstacles.generatePlan();
  }
  else {
    plan = findBall.generatePlan();
  }

  actuatorInterface->addToPlan(plan.getNextStep());
  */
  
  return true;
}


void FCFooDecision::onActuatorSensorData()
{
#ifdef RS_USE_DISPLAY
  theDisplay.setLastSkill(commandView->getLastServerString());
#endif
#ifdef RS_USE_OWN_LOGFILE
  logFile << gameView->getCurrentTime() << ": executed action "
	  << commandView->getLastServerString()
	  << std::endl;
  if ( commandView->getActualTurn(NOW) > AngleDeg(0) )
    logFile << gameView->getCurrentTime() << ": actual turn = "
	    << commandView->getActualTurn(NOW)
	    << std::endl;  
#endif
}


void
FCFooDecision::onAuralCoachSensorData()
{
  std::cerr << "I got " << agentView->getLastCoachMessage()
	    << " from the coach" << std::endl;
}


void
FCFooDecision::onAuralRefereeSensorData()
{
  std::cerr << "I got message from the referee." << std::endl;
}


void
FCFooDecision::onAuralPlayerSensorData()
{
  std::cerr << "I got " << agentView->getLastPlayerMessage()
	    << " from " << agentView->getLastPlayerMessageDirection()
	    << std::endl;
  ++aural_data;
}


void
FCFooDecision::onAuralSelfSensorData()
{
  std::cerr << "I got " << agentView->getLastSelfMessage()
	    << " from myself" << std::endl;
  ++aural_data;
}


void FCFooDecision::onInit()
{
  MoveCommand* move_cmd = new MoveCommand(Point(-10, -10));
  actuatorInterface->addToPlan(move_cmd);
}


void FCFooDecision::onVisualSensorData()
{
  //RS_LOG( LA_DECISION, gameView->getCurrentTime() << ": **saw**" );
}


void FCFooDecision::onAfterSensorData()
{
}


void FCFooDecision::onNewCycle(const bool forced)
{
  if ( forced ) {
    RS_LOG( LA_DECISION,
	    gameView->getCurrentTime() << ": **Forced NewCycle**" );
  }
  else {
    RS_LOG( LA_DECISION, gameView->getCurrentTime() << ": **NewCycle**" );
  }

#ifdef RS_USE_DISPLAY
  theDisplay.draw();
#endif

}


bool FCFooDecision::onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks)
{
  // We assume that if the sense_body is more than four ticks late
  // then it's lost.
  RS_LOG( LA_DECISION, "onEstimatedNewCycle(" << estimated_time
	  << ", " << ticks << ")" );

  // If more than nine sense_body's are missed in a row then assume that
  // the server is dead.
  if ( estimated_time > 9 )
    serverAlive = false;

  if ( ticks > 4 ) {
    // TODO: change this into sending a control command instead!
    //sensorInterface->ForceNewCycle();
    return true;
  } else
    return false;
}


void FCFooDecision::onCommandWarning()
{
  decide();
}


RS_END_NAMESPACE
