/* -*- Mode: C++ -*-
 * $Id: FindBall.cc,v 1.11 2001/09/29 18:55:20 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FindBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.11 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/09/29 18:55:20 $
 *
 *
 * HISTORY
 * $Log: FindBall.cc,v $
 * Revision 1.11  2001/09/29 18:55:20  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.10  2001/09/26 11:41:25  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.9  2001/08/30 22:56:15  frehe
 * Replaced TRUE with FuzzyBool(1.0) since the static consts are not initialized properly.
 *
 * Revision 1.8  2001/08/23 00:31:58  frehe
 * Removed dependencies.
 *
 * Revision 1.7  2001/01/24 07:00:39  olalu526
 * Fine tuning and fixing.
 *
 * Revision 1.6  2001/01/11 22:00:19  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.5  2000/10/07 15:07:36  matka134
 * Now using the predicate IsBallKnown
 *
 * Revision 1.4  2000/09/29 17:55:02  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.3  2000/09/28 18:52:17  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.2  2000/09/13 01:38:59  frehe
 * Updated the skills to use getView instead of static_cast<...>.
 *
 * Revision 1.1  2000/09/05 21:12:01  frehe
 * #include rs_log.h
 *
 */

/**
 * @file FindBall.cc
 * Implementation file for the skill class FindBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.11 $
 * $Author: frehe $
 * $Date: 2001/09/29 18:55:20 $
 */


#include "FindBall.h"
#include "BallView.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "IsBallKnown.h"
#include "GameView.h"


RS_BEGIN_NAMESPACE


FindBall::FindBall()
  : Skill(),
    ballView(NULL),
    gameView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
} 


FindBall::~FindBall()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
}


Plan FindBall::generatePlan()
{
  static Command* cmd;
  static AngleDegUnknown ang;
  static Plan plan;

  ang = ballView->getRelativeBallDirection(NOW);
  if ( ang.isUnknown() ) {
    cmd = new TurnCommand(45);
  }
  else
    cmd = new TurnCommand(ang.getDeg());

  plan = Plan(Step(cmd));
  delete cmd;
  cmd = NULL;
  
  return plan;
}


FuzzyBool FindBall::persistent()
{
  return FuzzyBool(0.0);
}


FuzzyBool FindBall::applicable()
{
  bool failed;

  /*
  RS::FuzzyBool f1(0.0);
  RS::FuzzyBool f2(0.2);
  RS::FuzzyBool f3(0.5);
  RS::FuzzyBool f4(0.6);
  RS::FuzzyBool f5(1.0);

  std::cout << "TRUE = " << TRUE << std::endl;
  std::cout << "FALSE = " << FALSE << std::endl;
  std::cout << "UNKNOWN = " << UNKNOWN << std::endl;
  
  if ( f1 == RS::FALSE ) {
    std::cout << "passed FuzzyBool(0.0) == FALSE" << std::endl;
  } else {
    std::cout << "failed FuzzyBool(0.0) == FALSE" << std::endl;
    failed = true;
  }

  if ( f3 == RS::UNKNOWN ) {
    std::cout << "passed FuzzyBool(0.5) == UNKNOWN" << std::endl;
  } else {
    std::cout << "failed FuzzyBool(0.5) == UNKNOWN" << std::endl;
    failed = true;
  }

  if ( f5 == RS::TRUE ) {
    std::cout << "passed FuzzyBool(1.0) == TRUE" << std::endl;
  } else {
    std::cout << "failed FuzzyBool(1.0) == TRUE" << std::endl;
    failed = true;
  }

  if ( !f1 == RS::TRUE ) {
    std::cout << "passed !FuzzyBool(0.0) == TRUE" << std::endl;
  } else {
    std::cout << "failed !FuzzyBool(0.0) == TRUE" << std::endl;
    failed = true;
  }

  if ( !f3 == RS::UNKNOWN ) {
    std::cout << "passed !FuzzyBool(0.5) == UNKNOWN" << std::endl;
  } else {
    std::cout << "failed !FuzzyBool(0.5) == UNKNOWN" << std::endl;
    failed = true;
  }

  if ( !f5 == RS::FALSE ) {
    std::cout << "passed !FuzzyBool(1.0) == FALSE" << std::endl;
  } else {
    std::cout << "failed !FuzzyBool(1.0) == FALSE" << std::endl;
    failed = true;
  }

  if ( !f1 ) {
    std::cout << "passed FuzzyBool(0.0) == false" << std::endl;
  } else {
    std::cout << "failed FuzzyBool(0.0) == false" << std::endl;
    failed = true;
  }

  if ( !f2 ) {
    std::cout << "passed FuzzyBool(0.2) == false" << std::endl;
  } else {
    std::cout << "failed FuzzyBool(0.2) == false" << std::endl;
    failed = true;
  }

  if ( !f3 ) {
    std::cout << "passed FuzzyBool(0.5) == false" << std::endl;
  } else {
    std::cout << "failed FuzzyBool(0.5) == false" << std::endl;
    failed = true;
  }

  if ( f3 ) {
    std::cout << "passed FuzzyBool(0.5) == true" << std::endl;
  } else {
    std::cout << "failed FuzzyBool(0.5) == true" << std::endl;
    failed = true;
  }

  if ( f4 ) {
    std::cout << "passed FuzzyBool(0.6) == true" << std::endl;
  } else {
    std::cout << "failed FuzzyBool(0.6) == true" << std::endl;
    failed = true;
  }

  if ( f5 ) {
    std::cout << "passed FuzzyBool(1.0) == true" << std::endl;
  } else {
    std::cout << "failed FuzzyBool(1.0) == true" << std::endl;
    failed = true;
  }
  */
  
  /*
  TimeUnknown t = ballView->getLastBallObservationTime();
  if ( ballView->getRelativeBallDirection(NOW).isUnknown() ||
       ballView->getBallPosition(NOW).isUnknown()) {
    return FuzzyBool(1.0);
  }
  else if (t.isUnknown()) {
    return FuzzyBool(1.0);
  }
  else if (t.getValue() < gameView->getCurrentTime() -2) {
    return FuzzyBool(1.0);
  }
  else {
    return FuzzyBool(0.0);
    }*/

  static IsBallKnown isBallKnown;
  return !isBallKnown();
}


RS_END_NAMESPACE
