/* -*- Mode: C++ -*-
 * $Id: GoaliePositioning.h,v 1.7 2001/10/10 18:33:44 aip11 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GoaliePositioning.h
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: aip11 $ $Date: 2001/10/10 18:33:44 $
 *
 *
 * HISTORY
 *
 * $Log: GoaliePositioning.h,v $
 * Revision 1.7  2001/10/10 18:33:44  aip11
 * Added some comment and tweaked a few constants
 *
 * Revision 1.6  2001/09/26 09:03:53  aip11
 * Fixed confidence bug
 *
 * Revision 1.4  2001/09/20 12:59:49  aip11
 * Somewhat working version
 *
 * Revision 1.3  2001/09/18 20:33:22  andbo928
 * Added view declarations to the header file so it compiles.
 *
 * Revision 1.2  2001/09/06 13:12:52  andbo928
 * Added some methods to the class. (SkillStub was bad)
 *
 * Revision 1.1  2001/09/05 14:41:03  stejo127
 * Added the GoaliePositioning skill.
 */

/**
 * @file GoaliePositioning.h
 * Include file for the skill class GoaliePositioning.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: aip11 $
 * $Date: 2001/10/10 18:33:44 $
 */


#ifndef GOALIEPOSITIONING_H
#define GOALIEPOSITIONING_H


#include "Skill.h"
#include "Plan.h"

RS_BEGIN_NAMESPACE

class ParameterView;
class BallView;
class AgentView;
class MarkerView;
class WorldStateView;
class TVector;

/**
 * @class GoaliePositioning GoaliePositioning.h
 * A skill for positioning the goalie between the goal and the ball in a smart way.
 * The goalie is placed on a half-circle centered in the centre of the goal if the ball isnt to close to the goal, then it will rush out and try to catch it. 
 * We decided to use a MoveToPosition of our own since the one in the skills-directory proved to be bad for our skill.
 * The skill also contains sub skills that is about equivalent to rush and catchball since they didnt work as well as we wanted.
 *
 * @pre Player must be a goalkeeper
 * @post The player is placed between the goal and the ball if the ball isnt too close to the goal in which case the goalie will rush out and try to catch it.
 *
 * @todo
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: aip11 $
 * $Date: 2001/10/10 18:33:44 $
 */
class GoaliePositioning : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  GoaliePositioning();

  /** Destructor should clean up after the skill. */
  virtual ~GoaliePositioning();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

private:
  /** Hide copy-constructor. */
  GoaliePositioning(const GoaliePositioning&);
  /** Hide assignment operator. */
  GoaliePositioning& operator= (const GoaliePositioning&);

protected:
  ParameterView* parameterView;
  BallView* ballView;
  AgentView* agentView;
  MarkerView* markerView;
  WorldStateView* worldStateView;
  /** Our moving-function */
  Plan MoveToPosition(const TVector& goal,const float power,float maxDist);
};


RS_END_NAMESPACE




#endif // !GOALIEPOSITIONING_H
