/* -*- Mode: C++ -*-
 * $Id: Display.h,v 1.3 2001/09/01 15:06:51 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Display.h
 * Unitname: Framework/test/TestTeam
 * $Revision: 1.3 $
 * Created by: Johan Thapper 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/09/01 15:06:51 $
 *
 *
 * HISTORY
 *
 * $Log: Display.h,v $
 * Revision 1.3  2001/09/01 15:06:51  frehe
 * Updated the file to work with the current version of RoboSoc.
 *
 * Revision 1.2  2000/09/16 18:39:40  frehe
 * Fixed the log messages.
 *
 * Revision 1.1  2000/09/16 18:37:06  frehe
 * Add a team that can be used as the basis for test teams.
 *
 */

/**
 * @file Display.h
 * Include file for the class Display.
 *
 * @par Unit:
 *      Framework/test/TestTeam
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/09/01 15:06:51 $
 */


#ifndef DISPLAY_H
#define DISPLAY_H


//#define RS_USE_COACH_DEBUGGING


#include <string>

#include "ViewManager.h"
#include "ParameterView.h"
#include "GameView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "PlayerView.h"
#ifdef RS_USE_COACH_DEBUGGING
#  include "CoachView.h"
#endif


#define CLEAR_SCREEN std::cout << char(27) << "[2J"
#define HOME_CURSOR std::cout << char(27) << "[H"
#define MOVE_CURSOR(x,y) std::cout << char(27) << '[' << x << ',' << y << 'H'


RS_BEGIN_NAMESPACE


/**
 * @class Display Display.h
 * Example class for what a debug information display could look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Framework/test/TestTeam
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/09/01 15:06:51 $
 */
class Display {
public:
  Display(UInt x = 40, UInt y = 10);
  ~Display();
  
  void setLastSkill(const std::string&);
  void draw();

protected:
  ViewManager* viewManager;
  ParameterView* parameterView;
  GameView* gameView;
  MarkerView* markerView;
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
#ifdef RS_USE_COACH_DEBUGGING
  CoachView* coachView;
#endif
  
  UInt col;
  UInt row;

  std::string lastSkill;
};


RS_END_NAMESPACE


#endif // !DISPLAY_H

