/*
 * $Id: TLine.h,v 1.1 2001/09/18 20:41:49 andbo928 Exp $
 * 
 * HISTORY 
 *
 * $Log: TLine.h,v $
 * Revision 1.1  2001/09/18 20:41:49  andbo928
 * Imported some sources from FuzzyFoo.
 *
 * Revision 1.4  2001/07/28 15:27:22  mikbr212
 * Added const
 *
 * Revision 1.3  2001/06/05 20:33:53  mikbr212
 * Fixed doxygen-stubs for all classes
 *
 */
#ifndef TLINE_H
#define TLINE_H

#include "TVector.h"
#include "library_types.h"

RS_BEGIN_NAMESPACE

/**
 * @class TLine TLine.h
 * Class for representating lines (Vector with origin).
 */

class TLine {
public:
  TLine(const TVector&, const TVector&); 
  TLine(const TVector&, const TVector&, const double&, bool=true); 
  ~TLine();

  bool isParallel(const TLine&) const;
  bool isThroughCircle(const TVector&, const double&) const;

  TVector getIntersection(const TLine&) const;
  TVector getIntersection(const TLine&, double&, double&) const;

  TVector getDistance(const TVector&) const;
  TVector getDistance(const TVector&, double&) const;
  
  bool extendToCircle(const TVector&, const double&);
  void extendToTLine(const TLine&);
  void extendToTLine(const TLine&, double&, double&);

  TVector getStart() const;
  TVector getEnd() const;
  TVector getVector() const;
  void setStart(const TVector&);
  void setVector(const TVector&);
  void setEnd(const TVector&);
  
protected:
  TVector p;
  TVector v;
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "TLine.icc"
#endif


#endif // !LINE_H
