/* -*- Mode: C++ -*-
 * $Id: WorldStateView.cc,v 1.3 2002/01/11 15:15:56 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WorldStateView.cc
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: frehe $ $Date: 2002/01/11 15:15:56 $
 *
 *
 * HISTORY
 *
 * $Log: WorldStateView.cc,v $
 * Revision 1.3  2002/01/11 15:15:56  frehe
 * Added std:: to make it compile.
 *
 * Revision 1.2  2001/09/26 13:02:21  frehe
 * Added some debug output for players.
 *
 * Revision 1.1  2001/09/18 20:41:50  andbo928
 * Imported some sources from FuzzyFoo.
 *
 * Revision 1.35  2001/07/30 15:16:44  andbo928
 * Now fixed to work with the new WorldState.
 *
 * Revision 1.34  2001/07/27 15:52:30  andbo928
 * Cosmetic changes.
 *
 * Revision 1.33  2001/07/16 15:42:34  andbo928
 * A nicer gauss-reduction.
 *
 * Revision 1.32  2001/07/12 12:49:38  andbo928
 * Now reduces observation confidence based on distance to observation.
 *
 * Revision 1.31  2001/07/10 17:28:25  andbo928
 * Did a little workaround to the stupid server-resends.
 *
 * Revision 1.30  2001/07/06 16:28:44  mikbr212
 * Fixed nextSeeCycle-phenomena
 *
 * Revision 1.29  2001/07/06 15:56:07  andbo928
 * Fixed a bug in myUpdateAfterSense.
 *
 * Revision 1.28  2001/07/06 10:00:39  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.27  2001/07/05 17:12:45  andbo928
 * Removed references to setTeam.
 *
 * Revision 1.26  2001/07/01 12:40:35  andbo928
 * Observations now merge onNewCycle
 *
 * Revision 1.25  2001/06/19 15:22:47  mikbr212
 * Now uses the team-value in the player
 *
 * Revision 1.24  2001/06/19 14:01:00  mikbr212
 * Fixed circular includes stuff. Somebody should check that I did it right.
 *
 * Revision 1.23  2001/04/24 20:43:17  mikbr212
 * added possession
 *
 * Revision 1.22  2001/04/24 17:33:57  andbo928
 * Fixed viewmode thingy.
 *
 * Revision 1.21  2001/04/24 17:11:42  mikbr212
 * added some reset
 *
 * Revision 1.20  2001/04/24 09:54:47  sigge
 * Moved communication policy to communication view. Communication view no longer erases messages not picked up from the incoming queue in order to be useful to skills.
 *
 * Revision 1.19  2001/04/23 10:22:18  mikbr212
 * Added uniformnumber on init
 *
 * Revision 1.18  2001/04/22 16:00:56  mikbr212
 * adjusted agent-speed direction
 *
 * Revision 1.17  2001/04/16 14:11:19  mikbr212
 * added magic ball stuff
 *
 * Revision 1.16  2001/04/15 23:23:06  andbo928
 * Fixed a bug when processing previous kicks.
 *
 * Revision 1.15  2001/04/15 00:08:46  andbo928
 * Now uses the new serialize() and deserialize() methods.
 *
 * Revision 1.14  2001/04/14 15:42:54  mikbr212
 * added kicks
 *
 * Revision 1.13  2001/04/14 10:33:44  andbo928
 * Removed doKick from BallState.
 *
 * Revision 1.12  2001/04/13 16:27:38  sigge
 * Added communication
 *
 * Revision 1.11  2001/04/12 21:55:46  andbo928
 * Merging now works better.
 *
 * Revision 1.10  2001/04/12 17:10:26  andbo928
 * Fixed the speedbug.
 *
 * Revision 1.9  2001/04/12 13:24:27  mikbr212
 * Enabled agent speed
 *
 * Revision 1.8  2001/04/12 13:22:57  mikbr212
 * Added sense and commands. TODO: check game mode
 *
 * Revision 1.7  2001/04/12 12:33:07  andbo928
 * Fixed things.
 *
 * Revision 1.6  2001/04/12 11:27:40  mikbr212
 * added some stuff
 *
 * Revision 1.5  2001/04/12 11:22:16  mikbr212
 * added some stuff
 *
 * Revision 1.4  2001/04/12 00:15:36  andbo928
 * The view now merges.
 *
 * Revision 1.3  2001/04/11 18:54:52  andbo928
 * Fixed some stuff.
 *
 * Revision 1.2  2001/04/11 18:19:25  mikbr212
 * Implemented the basics
 *
 * Revision 1.1  2001/04/11 09:10:25  mikbr212
 * Created view
 *
 *
 */

/**
 * @file WorldStateView.cc
 * Implementation file for the class WorldStateView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2002/01/11 15:15:56 $
 */

#include <stdio.h>
#include "WorldStateView.h"

// #include "CommunicationView.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "GameView.h"

#include "TurnNeckCommand.h"
#include "WorldStateView.h"
#include <string.h>
// #include "CommunicationView.h"

RS_BEGIN_NAMESPACE

WorldStateView::WorldStateView(const ViewId id)
  : View(id),
    ws(),
    obs(),
    lastWs(),
    newObservations(false),
    lastSeeTime(),
    lastSeeCycle(0),
    nextSeeCycle(0),
    lastCommand(NULL),
    lastTurnNeckCommand(NULL),
    obsTeammates(),
    obsOpponents(),
    parameterView(NULL),
    markerView(NULL),
    gameView(NULL)
  //    communicationView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(markerView, MARKER_VIEW_ID, parameterView);
  getView(gameView, GAME_VIEW_ID); //, DEFAULT_HISTORY_LENGTH);
  //  getView(communicationView, COMMUNICATION_VIEW_ID);
}

WorldStateView::~WorldStateView() {
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  //  viewManager->releaseView(COMMUNICATION_VIEW_ID);
}

void WorldStateView::myUpdateAfterInit() {
  AgentState agent = ws.getAgent();
  agent.setUniformNumber((int)sensorInterface->getUniformNumber());
  ws.setAgent(agent);
}

// NewCycle is called *before* the values in CSensorInterface
// are erased, so this is the last chance to use them...
void WorldStateView::myNewCycle() {
  if(newObservations)
  {
    float faceDir = obs.getAgent().getDirection() + obs.getAgent().getNeckDirection();
    
    // Check view mode.
    double delta = 0;
    if(sensorInterface)
    {
      switch (sensorInterface->getViewWidth())
      {
      case VW_UNKNOWN:
      case VW_NORMAL:
      default:
	delta = 45;
	break;
      case VW_WIDE:
	delta = 90;
	break;
      case VW_NARROW:
	delta = 22.5;
	break;
      }
    }
//      obs.print();
//      cout << obs.getBall().getSpeed() << endl;
    ws.merge(obs.getAgent().getPosition(),
	     faceDir - delta, faceDir + delta,
	     obs);
    //    cout << "ws.getAgent().getPosition() - " << ws.getAgent().getPosition() << endl;
    newObservations = false;
  }

  AgentState agent = ws.getAgent();
  if(lastCommand != NULL) {
    switch(lastCommand->getType()) {
    case CMD_Catch:
      // Do nothing, maybe set som penalty count?
      break;
    case CMD_Dash:
      {
      DashCommand* dashCmd = (DashCommand*) lastCommand;
      agent.doDash(dashCmd->getPower());
      lastDashBodyDir = agent.getDirection();
      }
      break;
    case CMD_Kick:
      {
      KickCommand* kickCmd = (KickCommand*) lastCommand;
      TVector kickVector =
	TVector::TPolarVector(kickCmd->getPower(),
			      kickCmd->getAngle().getDeg());
      BallState ball = ws.getBall();
      agent.doKick(kickVector, ball);
      ws.setBall(ball);
      }
      break;
    case CMD_Turn:
      {
      TurnCommand* turnCmd = (TurnCommand*) lastCommand;
      agent.doTurn(turnCmd->getAngle().getDeg());
      }
      break;
    case CMD_Move:
      {
      MoveCommand* moveCmd = (MoveCommand*) lastCommand;
      // Move seems to ignore confidence. fix it here.
      TVector tmp = TVector(moveCmd->getDestination());
      tmp.setC(1.0);
      agent.setPosition(tmp);
      agent.setSpeed(TVector(0,0));
      }
      break;
    default:
      break;
    }
    delete lastCommand;
    lastCommand = NULL;
  }

  if(lastTurnNeckCommand != NULL) {
    agent.doTurnNeck(lastTurnNeckCommand->getAngle().getDeg());
    delete lastTurnNeckCommand;
    lastTurnNeckCommand = NULL;
  }

  ws.setAgent(agent);
  lastWs = ws;

//    if(lastCommand != NULL) {
//      delete lastCommand;
//      lastCommand = NULL;
//    }
//    if(lastTurnNeckCommand != NULL) {
//      delete lastTurnNeckCommand;
//      lastTurnNeckCommand = NULL;
//    }
  ws.step();
//    if(nextSeeCycle < ws.getGameTime())
//      nextSeeCycle
}

void WorldStateView::myUpdateAfterSee() {

  gameView->updateAfterSee(currentUpdateNumber);
  markerView->updateAfterSee(currentUpdateNumber);
  
  // Compute next see cycle
  if(sensorInterface) {
    ViewWidth w(sensorInterface->getViewWidth());
    ViewQuality q(sensorInterface->getViewQuality());
    if(w == VW_NARROW ||
       (w == VW_NORMAL && q == VQ_LOW)) {
      // see info every cycle
      nextSeeCycle = ws.getGameTime() + 1;
      nextNextSeeCycle = nextSeeCycle + 1;
    } else if((w == VW_NORMAL && q == VQ_HIGH) ||
	      (w == VW_WIDE && q == VQ_LOW)) {
      // see info 2 of 3 cycles
      if(lastSeeCycle == ws.getGameTime() - 1) {
	nextSeeCycle = ws.getGameTime() + 2;
	nextNextSeeCycle = nextSeeCycle + 1;
      } else {
	nextSeeCycle = ws.getGameTime() + 1;
	nextNextSeeCycle = nextSeeCycle + 2;
      }
    } else if(w == VW_WIDE && q == VQ_HIGH) {
      // see info every 3rd cycle
      nextSeeCycle = ws.getGameTime() + 3;
      nextNextSeeCycle = nextSeeCycle + 3;
    } else {
      // huh? if something is unknown... should not happen
      nextSeeCycle = ws.getGameTime() + 1;
      nextNextSeeCycle = nextSeeCycle + 1;
    }
  } else {
    // eh? when do this happen?
    nextSeeCycle = ws.getGameTime() + 1;
    nextNextSeeCycle = nextSeeCycle + 1;
  }

  lastSeeTime = gameView->getCurrentTime();
  lastSeeCycle = ws.getGameTime();

  obs = WorldState();
  obs.setGameTime(lastSeeCycle);
  
  updateAgentAfterSee();
  updateBallAfterSee();
  updatePlayersAfterSee();

//    float faceDir = obs.getAgent().getDirection() + obs.getAgent().getNeckDirection();

  // Worldstate mergeing now occurs on newcycle.
  newObservations = true;
}

void WorldStateView::updateAgentAfterSee() {
  AgentState agent = ws.getAgent();

  bool faceDirComputed = false;

  // Compute body direction (face direction - current neck direction)
  // If no direction could be computed, use the old direction
  if(markerView->getNumberOfSeenLines() == 0) {
    RS_WARNING("Didn't see any lines! Face direction is unknown!");
  } else {
    LineObject line = markerView->getClosestSeenLine();
    AngleDegUnknown angle = line.getDirection();

    if(angle.isKnown()) {
      switch ( line.getLineId() ) {
      case SL_TOP:
	if ( angle > 0 ) {
	  angle = -angle;
	  break;
	}
	else {
	  angle = AngleDeg(180) - angle;
	  break;
	} 
	break;
      case SL_BOTTOM:
	if ( angle > 0 ) {
	  angle = AngleDeg(180) - angle;
	  break;
	}
	else {
	  angle = -angle;
	  break;
	}
	break;
      case SL_LEFT:
	if ( angle > 0 ) {
	  angle = AngleDeg(270) - angle;
	  break;
	}
	else if ( angle < 0 ) {
	  angle = AngleDeg(90) - angle;
	  break;
	} 
	else {
	  RS_WARNING("Couldn't compute face dir. Special case 1 is not treated.\n");
	  angle = AngleDegUnknown();
	  break;
	}
	break;
      case SL_RIGHT:
	if ( angle > 0 ) {
	  angle = AngleDeg(90) - angle;
	  break;
	}
	else if ( angle < 0 ) {
	  angle = AngleDeg(270) - angle;
	  break;
	} 
	else {
	  RS_WARNING("Couldn't compute face dir. Special case 2 is not treated.\n");
	  angle = AngleDegUnknown();
	  break;
	}
	break;
      default:
	RS_WARNING("Unknown line id: " + toString(line) + " in computeFaceDir.\n");
	angle = AngleDegUnknown();
	break;
      }
      
      if(angle.isKnown()) {
	agent.setDirection(angle.getDeg() - agent.getNeckDirection());
	faceDirComputed = true;
      }
    } 
  }

  // Compute position
  // Set confidence to 1 if position can be calculated.
  MarkerObject flag = markerView->getClosestSeenMarker();
  if(flag.getVector().isKnown()) {
    double faceDir = agent.getDirection() + agent.getNeckDirection();
    TVector flagVec = TVector(flag.getVector());

    TVector rel_pos = TVector::TPolarVector(flagVec.getLength(),
					    faceDir + flagVec.getAngle());

    TVector abs_pos(flag.getPosition());
    agent.setPosition((abs_pos - rel_pos).setC(1));

    // Compensate for playing on the other side
    if(faceDirComputed && gameView->getOurSide() == S_RIGHT) {
      agent.setPosition(-agent.getPosition());
      agent.setDirection(agent.getDirection() - 180);
    }
  }

  // Save agentState in obs
  agent.setGameTime(lastSeeCycle);
//    agent.setSpeed(TVector(0,0,1));
  obs.setAgent(agent);
}

TVector WorldStateView::computePosition(const float relDist, 
					const float relDir,
					const AgentState& agent) {

  TVector pos = TVector::TPolarVector(relDist, 
				      relDir 
				      + agent.getDirection()
				      + agent.getNeckDirection());
  pos = pos + agent.getPosition();

  // Gauss to the rescue.
  // confidence = 0.5 @ 83m
  pos.setC(exp(-0.00005 * relDist*relDist));

  return pos;
}

/**
 * @param dirChange in radians
 * @param relDir in angles
 */
TVector WorldStateView::computeSpeed(const float distChange, const float dirChange, 
				     const float relDist, const float relDir,
				     const AgentState& agent) {
  // Compute relative speed
  TVector unit = TVector::TPolarVector(1, relDir); // 
  float tmp = dirChange * relDist; // dch * r
  TVector speed(distChange * unit.getX() - tmp * unit.getY(),
		distChange * unit.getY() + tmp * unit.getX());
  
  // Convert to absolute speed
  speed.rotate(agent.getDirection() + agent.getNeckDirection());
  speed = speed.setC(1) + agent.getSpeed();
  
  // Gauss to the rescue.
  // confidence = 0.5 @ 83m
  speed.setC(exp(-0.00005 * relDist*relDist));

  return speed;
}

bool WorldStateView::magicBallSpeedComputation(BallState& ball, double dist) {
  if(lastWs.getBall().getC() > 0.5 && dist < parameterView->SP_kickable_area) {
    AgentState oldAgent = lastWs.getAgent();
    oldAgent.step();
    
    TVector framingError(obs.getAgent().getPosition() - oldAgent.getPosition());

    TVector speed(ball.getPosition() - (lastWs.getBall().getPosition() + framingError));
    speed.setC(1);
    ball.setSpeed(speed);
    return true;
  }
  return false;
}

void WorldStateView::updateBallAfterSee() {
  if(sensorInterface->sawBall()) {
    
    BallState ball;

    FloatUnknown dist = sensorInterface->getBallDistance();
    AngleDegUnknown dir = sensorInterface->getBallDirection();
    FloatUnknown distChange = sensorInterface->getBallDistanceChange();
    AngleDegUnknown dirChange = AngleDegUnknown(sensorInterface->getBallDirectionChange());

    // Compute position
    // Set confidence to 1 if position can be calculated
    if(dist.isKnown() && dir.isKnown()) 
    {
      ball.setPosition(computePosition(dist.getValue(), dir.getDeg(), obs.getAgent()));
      // Compute speed
      //        if(!magicBallSpeedComputation(ball,dist.getValue())) {
      
      if(distChange.isKnown() && dirChange.isKnown()) 
      {
	std::cout << distChange.getValue() << ", " << dirChange.getRad() << std::endl;
	ball.setSpeed(computeSpeed(distChange.getValue(),
				   dirChange.getRad(),
				   dist.getValue(),
				   dir.getDeg(),
				   obs.getAgent()));

//    	if(obs.getAgent().getUniformNumber() == 1)
//    	  cout << "BallSpeed known - " << ball.getSpeed().getLength() << endl;
      }
      else
      {
	ball.setSpeed(TVector(0,0,0));
//    	if(obs.getAgent().getUniformNumber() == 1)
//    	  cout << "BallSpeed unknown." << endl;
      }
    }
    else
    {
      ball.setC(0.0);
    }
    //      }
    // Save ballState in obs
    ball.setGameTime(lastSeeCycle);
    obs.setBall(ball);
  }
}

void WorldStateView::processPlayer(const ServerPlayerObject& plr) {
  // Ignore unknown players and compensate for bug in sensor interface
  if(plr.team == T_UNKNOWN)
    return;
  //    if((plr.team == T_OUR_TEAM && obsTeammates.size() <= 9) || 
  //       (plr.team == T_THEIR_TEAM && obsOpponents.size() <= 10)) { 

  PlayerState player;

  // Compute position
  if(plr.distance.isKnown() && plr.direction.isKnown()) 
  {
    player.setPosition(computePosition(plr.distance.getValue(),
				       plr.direction.getDeg(),
				       obs.getAgent()));
    
    // Compute speed
    if(plr.distanceChange.isKnown() && plr.directionChange.isKnown()) 
    {
      player.setSpeed(computeSpeed(plr.distanceChange.getValue(), 
				   TVector::degToRad(plr.directionChange.getValue()), // Convert to radians.
				   plr.distance.getValue(),
				   plr.direction.getDeg(),
				   obs.getAgent()));
    }
  } 
  else 
  {
    // If the position is not known, then this player is useless
    return; 
  }

  // Uniform number
  if(plr.uniformNumber.isKnown())
  {
    player.setUniformNumber(plr.uniformNumber.getValue());
  }
//    if(plr.team == T_OUR_TEAM)
//      cout << "tm: " << player.getUniformNumber();
//    else
//      cout << "op: " << player.getUniformNumber();
//    cout << " " << player.getPosition() << endl;

  player.setGameTime(lastSeeCycle);
  //      player.setTeam(plr.team);

  if(plr.team == T_OUR_TEAM)
  {
//      vector<PlayerState>::iterator it;
//      for(it = obsTeammates.begin(); it != obsTeammates.end(); it++)
//      {
//        if( it->getUniformNumber() == player.getUniformNumber() 
//  	  && it->getUniformNumber() != -1 )
//        {
//  	// FIXME: This should probably be smarter.
//  	// If we detect an duplicate observation we
//  	// invalidate all previous observations.
//  	obsTeammates.clear();
//  	break;
//        }
//      }
    obsTeammates.push_back(player);
  }
  else 
  {
//      vector<PlayerState>::iterator it;
//      for(it = obsOpponents.begin(); it != obsOpponents.end(); it++)
//      {
//        if( it->getUniformNumber() == player.getUniformNumber() 
//  	  && it->getUniformNumber() != -1 )
//        {
//  	// FIXME: This should probably be smarter.
//  	// If we detect an duplicate observation we
//  	// invalidate all previous observations.
//  	obsOpponents.clear();
//  	break;
//        }
//      }
    obsOpponents.push_back(player);
  }
}

void WorldStateView::updatePlayersAfterSee() {
  obsTeammates.clear();
  obsOpponents.clear();

  sensorInterface->forEachPlayer(&View::processPlayer, this);

  RS_LOG( LA_PLAYER_VIEW, "#Teammates " << obsTeammates.size() );
  RS_LOG( LA_PLAYER_VIEW, "#Opponents " << obsOpponents.size() );
  
  obs.setTeammates(obsTeammates);
  obs.setOpponents(obsOpponents);
}

void WorldStateView::myUpdateAfterHear()
{

  // FIXME: Reenable if communications are enabled.

//    communicationView->updateAfterHear(currentUpdateNumber);
//    gameView->updateAfterHear(currentUpdateNumber);

//    vector<Message> messages = communicationView->getMessages(worldStateMessageType);
//    if(messages.size() > 1)
//      cout << "Expected at most one worldStateMessage, but got " << messages.size() << endl;
//    if(messages.size() > 0)
//    {
//      WorldState recievedWS;
//      recievedWS.deserialize(messages[0].getContents());
//      //      delete[] fulbuffert;

//      //    recievedWS.step();

//  //      // FIXME: Merge with current world state
//  //      cout << "Remote worldstate recieved:" << endl;
//  //      cout << "GameTime: " << recievedWS.getGameTime() 
//  //  	 << " previous: " << ws.getGameTime() << endl;
//  //      cout << "Agent: " 
//  //  	 << recievedWS.getAgent().getPosition() << ", "
//  //  	 << recievedWS.getAgent().getSpeed() << endl;

//  //      cout << "Ball: "
//  //  	 << recievedWS.getBall().getPosition() << ", "
//  //  	 << recievedWS.getBall().getSpeed() << endl;

//  //      cout << "Num tm: " << recievedWS.getTeammates().size() 
//  //  	 << ", num op: "  << recievedWS.getOpponents().size() << endl;

//      ws.merge(recievedWS);
//  //     ws.print();
//    }
}

void WorldStateView::myUpdateAfterCommand()
{
  /* Since we currently send all commands last in the cycle, even
     immediate actions, we can do all the updating at the begining
     of the new cycle. */

  if(sensorInterface->getLastCommand() != NULL) {
    ServerCommand* cmd = sensorInterface->getLastCommand()->clone();

    switch(cmd->getType()) {
    case CMD_Catch:
    case CMD_Dash:
    case CMD_Kick:
    case CMD_Turn:
    case CMD_Move:
      if(lastCommand == NULL)
	lastCommand = cmd;
      else
	RS_WARNING("WorldStateView: Got several commands this cycle!");
      break;
    case CMD_TurnNeck:
      if(lastTurnNeckCommand == NULL)
	lastTurnNeckCommand = (TurnNeckCommand*) cmd;
      else
	RS_WARNING("WorldStateView: Got several TurnNeckCommands this cycle!");
      break;
    default:
      break;
    }
  }
}

void WorldStateView::myUpdateAfterSense()
{
  AgentState agent = ws.getAgent();
  agent.setNeckDirection(sensorInterface->getHeadDirection().getDeg());

  agent.setSpeed(TVector::TPolarVector(sensorInterface->getSpeed(),
  				       lastDashBodyDir));
  
  // FIXME: Check if this is better? (sserver 7 gives the direction also...)
  //  agent.setSpeed(TVector(sensorInterface->getAgentSpeed()));

  agent.setStamina(sensorInterface->getStamina());
  agent.setEffort(sensorInterface->getEffort());
  agent.setUniformNumber(sensorInterface->getUniformNumber());

  ws.setAgent(agent);
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "WorldStateView.icc"
#endif
